--- old/usr/src/uts/common/os/kmem.c	2015-05-27 16:12:07.133130502 -0400
+++ new/usr/src/uts/common/os/kmem.c	2015-05-27 16:12:07.141163634 -0400
@@ -20,6 +20,7 @@
  */
 /*
  * Copyright (c) 1994, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright 2015 Nexenta Systems, Inc.  All rights reserved.
  */
 
 /*
@@ -2169,6 +2170,21 @@
 }
 
 /*
+ * Set the working set statistics for cp's depot to zero.  (Everything is
+ * eligible for reaping.)
+ */
+static void
+kmem_depot_ws_zero(kmem_cache_t *cp)
+{
+	mutex_enter(&cp->cache_depot_lock);
+	cp->cache_full.ml_reaplimit = cp->cache_full.ml_total;
+	cp->cache_full.ml_min = cp->cache_full.ml_total;
+	cp->cache_empty.ml_reaplimit = cp->cache_empty.ml_total;
+	cp->cache_empty.ml_min = cp->cache_empty.ml_total;
+	mutex_exit(&cp->cache_depot_lock);
+}
+
+/*
  * Reap all magazines that have fallen out of the depot's working set.
  */
 static void
@@ -3238,14 +3254,7 @@
 			kmem_magazine_destroy(cp, pmp, prounds);
 	}
 
-	/*
-	 * Updating the working set statistics twice in a row has the
-	 * effect of setting the working set size to zero, so everything
-	 * is eligible for reaping.
-	 */
-	kmem_depot_ws_update(cp);
-	kmem_depot_ws_update(cp);
-
+	kmem_depot_ws_zero(cp);
 	kmem_depot_ws_reap(cp);
 }
 
@@ -3270,16 +3279,14 @@
 }
 
 /*
- * Reap (almost) everything right now.  See kmem_cache_magazine_purge()
- * for explanation of the back-to-back kmem_depot_ws_update() calls.
+ * Reap (almost) everything right now.
  */
 void
 kmem_cache_reap_now(kmem_cache_t *cp)
 {
 	ASSERT(list_link_active(&cp->cache_link));
 
-	kmem_depot_ws_update(cp);
-	kmem_depot_ws_update(cp);
+	kmem_depot_ws_zero(cp);
 
 	(void) taskq_dispatch(kmem_taskq,
 	    (task_func_t *)kmem_depot_ws_reap, cp, TQ_SLEEP);
